{% extends "base.html" %}
{% load i18n %}

{% block content %}
  <h1>{% trans "Weather conditions at " %}{{ object.station.name }}</h1>
  <div class="panel panel-default">
    <div class="panel-heading">{% trans "Latest measurements" %}</div>
    <div class="panel-body">
      {% include "synopticgroupstation-report.html" %}
    </div>
  </div>
  {% for synoptic_timeseries in object.synoptictimeseries_set.primary %}
    <h2>{{ synoptic_timeseries.get_title }}</h2>
    <img src="../../chart/{{ synoptic_timeseries.id }}.png">
    <hr>
  {% endfor %}
{% endblock %}
